/*
#
# This attempts to watch drv2.so calls as they happen
#
# $Id:$
#
# Copyright (C) 2002 Kees Cook
# kees@outflux.net, http://outflux.net/
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
# http://www.gnu.org/copyleft/gpl.html
#
*/

#include <dlfcn.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <sys/time.h>

#include "codec.h"

char * pn_error(unsigned int errnum)
{
        static char buf[128];

        switch (errnum)
        {
        case 0x80004001:
                return "Not Implemented";
        case 0x8007000E:
                return "Out of Memory";
        case 0x80070057:
                return "Invalid Argument";
        case 0x80004002:
                return "No Interface";
        case 0x80004003:
                return "Bad Pointer";
        case 0x80070006:
                return "Bad? Handle";
        case 0x80004004:
                return "Abort";
        case 0x80004005:
                return "Fail";
        case 0x80070005:
                return "Access Denied";
        case 0x0:
                return "Success";
        default:
                sprintf(buf,"Unknown error: 0x%x",errnum);
                return buf;
        }
}

// prototypes...
unsigned int (*rv20_init)(unsigned int * info, void ** handle);
unsigned int (*rv20_hive)(unsigned int * info, void * handle);
unsigned int (*rv20_free)(void * handle);
unsigned int (*rv20_custom)(unsigned int * info, void * handle);
unsigned int (*rv20_transform)
              (
                unsigned int input_size,  // +8       read size?
                unsigned int output_size,  // +0       write size?
                unsigned char * input,// +10      pointer test, readable
                unsigned char * output, // +14      pointer test, writable
                void * handle  // +18 esi  source? handle?
              );

void * rv20;

int _init()
{
        if (!(rv20=dlopen("/kees/projects/so_load/rp/drv2.so.6.0",RTLD_LAZY)))
        {
                fprintf(stderr,"dlopen: %s\n",dlerror());
                return(1);
        }
                                
        if (!((void*)rv20_init=dlsym(rv20,"RV20toYUV420Init")))
        {
                fprintf(stderr,"dlsym('%s'): %s\n",
                                "RV20toYUV420Init",dlerror());
                return(1);
        }

        if (!((void*)rv20_free=dlsym(rv20,"RV20toYUV420Free")))
        {
                fprintf(stderr,"dlsym('%s'): %s\n",
                                "RV20toYUV420Free",dlerror());
                return(1);
        }

        if (!((void*)rv20_custom=dlsym(rv20,"RV20toYUV420CustomMessage")))
        {
                fprintf(stderr,"dlsym('%s'): %s\n",
                                "RV20toYUV420CustomMessage",dlerror());
                return(1);
        }

        if (!((void*)rv20_hive=dlsym(rv20,"RV20toYUV420HiveMessage")))
        {
                fprintf(stderr,"dlsym('%s'): %s\n",
                                "RV20toYUV420HiveMessage",dlerror());
                return(1);
        }

        if (!((void*)rv20_transform=dlsym(rv20,"RV20toYUV420Transform")))
        {
                fprintf(stderr,"dlsym('%s'): %s\n",
                                "RV20toYUV420Transform",dlerror());
                return(1);
        }

        printf("Wedge Loaded.\n");

        return 0;
}

int _fini()
{
        dlclose(rv20);

        printf("Wedge Unloaded.\n");

        return 0;
}

void dumpmem(char * name, void * addr, int len, int printable)
{
        int i;
        int loc=0;
        unsigned char * address=(unsigned char *)addr;

        if (!address)
        {
                fprintf(stderr,"%s: NULL\n",name);
                return;
        }

        fprintf(stderr,"%s:\n",name);

        for (i=0;i<len;i++)
        {
                if (printable && isprint(address[i]))
                        fprintf(stderr,"%c ",address[i]);
                else
                        fprintf(stderr,"0x%02x, ",address[i]);
                loc++;

                if (loc==8)
                {
                        fprintf(stderr,"\n");
                        loc=0;
                }
        }
        if (loc!=0)
                fprintf(stderr,"\n");
}




void * rv20;

unsigned int RV20toYUV420CustomMessage(
                unsigned int * info,
                void * handle
                )
{
        unsigned int result;

        fprintf(stderr,"%s: info: %x handle: %x\n",__FUNCTION__,
                        (unsigned int)info,(unsigned int)handle);

        result=0x80004001;

        /*
        dumpmem("one",one,4,1);
        dumpmem("two",two,4,1);
        dumpmem("three",three,4,1);
        dumpmem("four",four,4,1);
        return 1;
        */
        result=
        rv20_custom(info,handle);

        fprintf(stderr,"%s: returning %s\n",__FUNCTION__,pn_error(result));

        return result;
}

unsigned int RV20toYUV420Free(void * handle)
{
        unsigned int result;

        fprintf(stderr,"%s: handle: %x\n",__FUNCTION__,
                        (unsigned int)handle);

        /*
        dumpmem("one",one,4,1);
        dumpmem("two",two,4,1);
        dumpmem("three",three,4,1);
        dumpmem("four",four,4,1);
        */

        result=rv20_free(handle);

        fprintf(stderr,"%s: returning %s\n",__FUNCTION__,pn_error(result));

        return result;
}

unsigned int RV20toYUV420HiveMessage(unsigned int * info, void * handle)
{
        unsigned int result;

        fprintf(stderr,"%s: info: %x\n",__FUNCTION__,
                        (unsigned int)info);
        fprintf(stderr,"%s: handle: %x\n",__FUNCTION__,
                        (unsigned int)handle);

        {
                int i;
                for (i=0;i<100/4;i++)
                        printf("int info[%02d]: 0x%x\n",i,info[i]);
                dumpmem("one",info,100,0);
        }

        result=rv20_hive(info,handle);

        fprintf(stderr,"%s: returning %s\n",__FUNCTION__,pn_error(result));

        return result;
}

// This appears to return void from the disassembly
unsigned int RV20toYUV420Init(unsigned int * info, void * * handle)
{
        unsigned int result;

        struct init_t * init;

        init=(void*)info;

        printf("%s: unknown short[00]: 0x%04x\n",__FUNCTION__,init->bee);
        printf("%s: dimensions: %dx%d\n",__FUNCTION__,init->width,init->height);
        printf("%s: unknown short[01]: 0x%04x\n",__FUNCTION__,init->zero);
        printf("%s: callback: 0x%x\n",__FUNCTION__,(unsigned int)init->callback);
        /*
        {
                int i;
                for (i=0;i<4;i++)
                        printf("unknown short[%02d]: 0x%x\n",i+2,init->unknown[i]);
        }
        */
        printf("%s: H.263 ID: 0x%x\n",__FUNCTION__,init->h_id);

        /*
        if (info)
        {
                int i;
                for (i=0;i<11;i++)
                        fprintf(stderr,"int info[%02d]: 0x%012x\n",i,info[i]);
        }

        dumpmem("info",info,100,0);

        if (init->callback)
        {
                int cool=0;
                printf("calling...\n");
                cool=init->callback();
                printf("back: %d\n",cool);
        }
        */

        result=rv20_init(info,handle);

        printf("%s: %s (*handle: 0x%x)\n",__FUNCTION__,
                        pn_error(result),handle ? *((unsigned int*)handle) : 0);

        return result;
}

int calls=0;
struct timeval begintime;
unsigned int RV20toYUV420Transform(
                unsigned int input_size,  // +8       read size?
                unsigned int output_size,  // +0       write size?
                unsigned char * input,// +10      pointer test, readable
                unsigned char * output, // +14      pointer test, writable
                void * handle  // +18 esi  source? handle?
                )
{
        struct timeval nowtime;
        unsigned int result;
        long msec;
        long sec;

        if (calls==0)
        {
                gettimeofday(&begintime,NULL);
                memcpy(&nowtime,&begintime,sizeof(nowtime));
        }
        else
                gettimeofday(&nowtime,NULL);

        msec=nowtime.tv_usec-begintime.tv_usec;
        if (msec<0)
        {
                msec+=1000000;
                nowtime.tv_sec--;
        }
        sec=nowtime.tv_sec-begintime.tv_sec;
        msec/=1000;
        
        fprintf(stderr,"%s: (%d frames, %02ld.%03lds == %ld fps)\n",
                        __FUNCTION__,calls++,
                        sec,msec,sec ? calls/sec : 0);

        fprintf(stderr,"%s: input_size: %x\n",__FUNCTION__,
                        (unsigned int)input_size);
        fprintf(stderr,"%s: output_size: %x\n",__FUNCTION__,
                        (unsigned int)output_size);
        fprintf(stderr,"%s: input: %x\n",__FUNCTION__,
                        (unsigned int)input);
        fprintf(stderr,"%s: output: %x\n",__FUNCTION__,
                        (unsigned int)output);
        fprintf(stderr,"%s: handle: %x\n",__FUNCTION__,
                        (unsigned int)handle);

        result=rv20_transform(input_size,output_size,input,output,handle);

        fprintf(stderr,"%s: returning %s\n",__FUNCTION__,pn_error(result));

        return result;
}


